# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc_stubs/tunnel_iface.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dgrpc_stubs/tunnel_iface.proto\x12\x0ctunnel_iface\"q\n\x0eResponseHeader\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x1a\n\rresponse_code\x18\x03 \x01(\x05H\x00\x88\x01\x01\x12\x14\n\x07message\x18\x04 \x01(\tH\x01\x88\x01\x01\x42\x10\n\x0e_response_codeB\n\n\x08_message\"#\n\x10TestCommsRequest\x12\x0f\n\x07message\x18\x01 \x01(\t\"\\\n\x11TestCommsResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.tunnel_iface.ResponseHeader\x12\x10\n\x08response\x18\x02 \x01(\t\"\x9e\x02\n\rTunnelRequest\x12\x14\n\x07\x61\x64\x64ress\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08protocol\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x14\n\x07timeout\x18\x03 \x01(\x05H\x02\x88\x01\x01\x12\x15\n\x08username\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x15\n\x08password\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x13\n\x06\x64omain\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x18\n\x0bremote_addr\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x13\n\x0b\x61llow_cidrs\x18\x08 \x03(\tB\n\n\x08_addressB\x0b\n\t_protocolB\n\n\x08_timeoutB\x0b\n\t_usernameB\x0b\n\t_passwordB\t\n\x07_domainB\x0e\n\x0c_remote_addr\"L\n\x0eTunnelResponse\x12\x14\n\x07\x61\x64\x64ress\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x10\n\x03url\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\n\n\x08_addressB\x06\n\x04_url\"@\n\x11OpenTunnelRequest\x12+\n\x06tunnel\x18\x01 \x01(\x0b\x32\x1b.tunnel_iface.TunnelRequest\"P\n\x12\x43loseTunnelRequest\x12\x14\n\x07\x61\x64\x64ress\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x10\n\x03url\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\n\n\x08_addressB\x06\n\x04_url\"\x18\n\x16\x43loseAllTunnelsRequest\"N\n\x10GetTunnelRequest\x12\x14\n\x07\x61\x64\x64ress\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x10\n\x03url\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\n\n\x08_addressB\x06\n\x04_url\"\x14\n\x12ListTunnelsRequest\"v\n\x12OpenTunnelResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.tunnel_iface.ResponseHeader\x12\x0f\n\x07success\x18\x02 \x01(\x08\x12\x10\n\x03url\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x06\n\x04_url\"]\n\x13\x43loseTunnelResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.tunnel_iface.ResponseHeader\x12\x0f\n\x07success\x18\x02 \x01(\x08\"a\n\x17\x43loseAllTunnelsResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.tunnel_iface.ResponseHeader\x12\x0f\n\x07success\x18\x02 \x01(\x08\"x\n\x11GetTunnelResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.tunnel_iface.ResponseHeader\x12,\n\x06tunnel\x18\x02 \x01(\x0b\x32\x1c.tunnel_iface.TunnelResponse\"{\n\x13ListTunnelsResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.tunnel_iface.ResponseHeader\x12-\n\x07tunnels\x18\x02 \x03(\x0b\x32\x1c.tunnel_iface.TunnelResponse\"\x14\n\x12SyncTunnelsRequest\"]\n\x13SyncTunnelsResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.tunnel_iface.ResponseHeader\x12\x0f\n\x07success\x18\x02 \x01(\x08\x32\xe8\x04\n\x0fTunnelInterface\x12N\n\tTestComms\x12\x1e.tunnel_iface.TestCommsRequest\x1a\x1f.tunnel_iface.TestCommsResponse\"\x00\x12Q\n\nOpenTunnel\x12\x1f.tunnel_iface.OpenTunnelRequest\x1a .tunnel_iface.OpenTunnelResponse\"\x00\x12T\n\x0b\x43loseTunnel\x12 .tunnel_iface.CloseTunnelRequest\x1a!.tunnel_iface.CloseTunnelResponse\"\x00\x12`\n\x0f\x43loseAllTunnels\x12$.tunnel_iface.CloseAllTunnelsRequest\x1a%.tunnel_iface.CloseAllTunnelsResponse\"\x00\x12N\n\tGetTunnel\x12\x1e.tunnel_iface.GetTunnelRequest\x1a\x1f.tunnel_iface.GetTunnelResponse\"\x00\x12T\n\x0bSyncTunnels\x12 .tunnel_iface.SyncTunnelsRequest\x1a!.tunnel_iface.SyncTunnelsResponse\"\x00\x12T\n\x0bListTunnels\x12 .tunnel_iface.ListTunnelsRequest\x1a!.tunnel_iface.ListTunnelsResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc_stubs.tunnel_iface_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_RESPONSEHEADER']._serialized_start=47
  _globals['_RESPONSEHEADER']._serialized_end=160
  _globals['_TESTCOMMSREQUEST']._serialized_start=162
  _globals['_TESTCOMMSREQUEST']._serialized_end=197
  _globals['_TESTCOMMSRESPONSE']._serialized_start=199
  _globals['_TESTCOMMSRESPONSE']._serialized_end=291
  _globals['_TUNNELREQUEST']._serialized_start=294
  _globals['_TUNNELREQUEST']._serialized_end=580
  _globals['_TUNNELRESPONSE']._serialized_start=582
  _globals['_TUNNELRESPONSE']._serialized_end=658
  _globals['_OPENTUNNELREQUEST']._serialized_start=660
  _globals['_OPENTUNNELREQUEST']._serialized_end=724
  _globals['_CLOSETUNNELREQUEST']._serialized_start=726
  _globals['_CLOSETUNNELREQUEST']._serialized_end=806
  _globals['_CLOSEALLTUNNELSREQUEST']._serialized_start=808
  _globals['_CLOSEALLTUNNELSREQUEST']._serialized_end=832
  _globals['_GETTUNNELREQUEST']._serialized_start=834
  _globals['_GETTUNNELREQUEST']._serialized_end=912
  _globals['_LISTTUNNELSREQUEST']._serialized_start=914
  _globals['_LISTTUNNELSREQUEST']._serialized_end=934
  _globals['_OPENTUNNELRESPONSE']._serialized_start=936
  _globals['_OPENTUNNELRESPONSE']._serialized_end=1054
  _globals['_CLOSETUNNELRESPONSE']._serialized_start=1056
  _globals['_CLOSETUNNELRESPONSE']._serialized_end=1149
  _globals['_CLOSEALLTUNNELSRESPONSE']._serialized_start=1151
  _globals['_CLOSEALLTUNNELSRESPONSE']._serialized_end=1248
  _globals['_GETTUNNELRESPONSE']._serialized_start=1250
  _globals['_GETTUNNELRESPONSE']._serialized_end=1370
  _globals['_LISTTUNNELSRESPONSE']._serialized_start=1372
  _globals['_LISTTUNNELSRESPONSE']._serialized_end=1495
  _globals['_SYNCTUNNELSREQUEST']._serialized_start=1497
  _globals['_SYNCTUNNELSREQUEST']._serialized_end=1517
  _globals['_SYNCTUNNELSRESPONSE']._serialized_start=1519
  _globals['_SYNCTUNNELSRESPONSE']._serialized_end=1612
  _globals['_TUNNELINTERFACE']._serialized_start=1615
  _globals['_TUNNELINTERFACE']._serialized_end=2231
# @@protoc_insertion_point(module_scope)
