# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc_stubs/camera_iface.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dgrpc_stubs/camera_iface.proto\x12\x0c\x63\x61mera_iface\"H\n\x0eStreamSettings\x12\x10\n\x03\x66ps\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12\x12\n\x05scale\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x06\n\x04_fpsB\x08\n\x06_scale\"q\n\x0eResponseHeader\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x1a\n\rresponse_code\x18\x02 \x01(\x05H\x00\x88\x01\x01\x12\x14\n\x07message\x18\x03 \x01(\tH\x01\x88\x01\x01\x42\x10\n\x0e_response_codeB\n\n\x08_message\"#\n\x10TestCommsRequest\x12\x0f\n\x07message\x18\x01 \x01(\t\"\\\n\x11TestCommsResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.camera_iface.ResponseHeader\x12\x10\n\x08response\x18\x02 \x01(\t\"\xad\x02\n\x0fSnapshotRequest\x12\x10\n\x08rtsp_uri\x18\x01 \x01(\t\x12\x1a\n\rsnapshot_type\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x1c\n\x0fsnapshot_length\x18\x03 \x01(\x05H\x01\x88\x01\x01\x12\x1b\n\x0eoutput_channel\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\x18\n\x0b\x63\x61mera_name\x18\x05 \x01(\tH\x03\x88\x01\x01\x12:\n\x0fstream_settings\x18\x06 \x01(\x0b\x32\x1c.camera_iface.StreamSettingsH\x04\x88\x01\x01\x42\x10\n\x0e_snapshot_typeB\x12\n\x10_snapshot_lengthB\x11\n\x0f_output_channelB\x0e\n\x0c_camera_nameB\x12\n\x10_stream_settings\"m\n\x10SnapshotResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.camera_iface.ResponseHeader\x12\x15\n\x08response\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x0b\n\t_response\"\x1a\n\x18IsSnapshotRunningRequest\"z\n\x19IsSnapshotRunningResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.camera_iface.ResponseHeader\x12\x17\n\nis_running\x18\x02 \x01(\x08H\x00\x88\x01\x01\x42\r\n\x0b_is_running2\x95\x02\n\x0b\x63\x61meraIface\x12N\n\tTestComms\x12\x1e.camera_iface.TestCommsRequest\x1a\x1f.camera_iface.TestCommsResponse\"\x00\x12N\n\x0bRunSnapshot\x12\x1d.camera_iface.SnapshotRequest\x1a\x1e.camera_iface.SnapshotResponse\"\x00\x12\x66\n\x11IsSnapshotRunning\x12&.camera_iface.IsSnapshotRunningRequest\x1a\'.camera_iface.IsSnapshotRunningResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc_stubs.camera_iface_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_STREAMSETTINGS']._serialized_start=47
  _globals['_STREAMSETTINGS']._serialized_end=119
  _globals['_RESPONSEHEADER']._serialized_start=121
  _globals['_RESPONSEHEADER']._serialized_end=234
  _globals['_TESTCOMMSREQUEST']._serialized_start=236
  _globals['_TESTCOMMSREQUEST']._serialized_end=271
  _globals['_TESTCOMMSRESPONSE']._serialized_start=273
  _globals['_TESTCOMMSRESPONSE']._serialized_end=365
  _globals['_SNAPSHOTREQUEST']._serialized_start=368
  _globals['_SNAPSHOTREQUEST']._serialized_end=669
  _globals['_SNAPSHOTRESPONSE']._serialized_start=671
  _globals['_SNAPSHOTRESPONSE']._serialized_end=780
  _globals['_ISSNAPSHOTRUNNINGREQUEST']._serialized_start=782
  _globals['_ISSNAPSHOTRUNNINGREQUEST']._serialized_end=808
  _globals['_ISSNAPSHOTRUNNINGRESPONSE']._serialized_start=810
  _globals['_ISSNAPSHOTRUNNINGRESPONSE']._serialized_end=932
  _globals['_CAMERAIFACE']._serialized_start=935
  _globals['_CAMERAIFACE']._serialized_end=1212
# @@protoc_insertion_point(module_scope)
